/* -*- c-file-style: "ruby"; indent-tabs-mode: nil -*- */
/**********************************************************************

  rbgpui.h -

  $Author: ktou $
  $Date: 2006/01/19 03:10:28 $

  Copyright (C) 2005 Ruby-GNOME2 Project Team
  Copyright (C) 2004 Kouhei Sutou <kou@cozmixng.org>

**********************************************************************/

#ifndef RBGPUI_H
#define RBGPUI_H

#include "rblibgnomeprintuiversion.h"

#include <ruby.h>

#include <rbgtk.h>

#define RBGPUI_MAJOR_VERSION RBGLIB_MAJOR_VERSION
#define RBGPUI_MINOR_VERSION RBGLIB_MINOR_VERSION
#define RBGPUI_MICRO_VERSION RBGLIB_MICRO_VERSION

#endif
