EXPORTS
	Init_glib2
	mGLib		DATA
	rbgobj_cType	DATA
	rbgobj_id_children DATA
	rbgobj_add_relative
	rbgobj_add_relative_removable
	rbgobj_get_relative_removable
	rbgobj_remove_relative
	rbgobj_remove_relative_all
	rbgobj_define_class
	rbgobj_define_class_dynamic
	rbgobj_instance_from_ruby_object
	rbgobj_ruby_object_from_instance
	rbgobj_ruby_object_from_instance2
	rbgobj_ruby_object_from_instance_with_unref
	rbgobj_create_object
	rbgobj_gobject_new
	rbgobj_boxed_create
	rbgobj_boxed_get
	rbgobj_boxed_not_copy_obj
	rbgobj_make_boxed
	rbgobj_add_abstract_but_create_instance_class
	rbgobj_gtype_get
	rbgobj_gtype_new
	rbgobj_gvalue_to_rvalue
	rbgobj_rvalue_to_gvalue
	rbgobj_initialize_object
	rbgobj_lookup_class
	rbgobj_lookup_class_by_gtype
	rbgobj_ptr2cptr
	rbgobj_ptr_new
	rbgobj_make_enum
	rbgobj_get_enum
	rbgobj_make_flags
	rbgobj_get_flags
	rbgobj_register_g2r_func
	rbgobj_register_r2g_func
	rbgobj_register_class
	rbgobj_register_property_getter
	rbgobj_register_property_setter
	rbgobj_set_signal_func
	rbgobj_get_signal_func
	rbgobj_add_constants
	rbgobj_constant_remap
	rbgobj_signal_wrap
	g_rclosure_new
	g_rclosure_attach
	rbgobj_ruby_value_get_type
	g_value_get_ruby_value
	g_value_set_ruby_value
	rbg_cstr2rval
	rbg_cstr2rval_with_free
	rbg_string_value_ptr
        rbg_set_internal_poll_func
        rbg_remove_internal_poll_func
	rbgutil_id_module_eval		DATA
	rbgutil_def_setters
	rbgutil_glist2ary
	rbgutil_glist2ary_boxed
	rbgutil_glist2ary_and_free
	rbgutil_glist2ary_boxed_and_free
	rbgutil_gslist2ary
	rbgutil_gslist2ary_boxed
	rbgutil_gslist2ary_and_free
	rbgutil_gslist2ary_boxed_and_free
	rbgutil_set_properties
	rbgutil_sym_g2r_func
	rbgutil_protect
	rbgerr_define_gerror
	rbgerr_gerror2exception
	rbgobj_fund_define_fundamental
	rbgobj_gc_mark_gvalue
	rbgobj_gc_mark_instance
